@@ -25,14 +25,20 @@ class PartitionNode(NodeBase):
     flag_boot = 1
     flag_primary = 2
 
-    def __init__(self, config, parent, prev_partition):
+    def __init__(self, config, state, parent, prev_partition):
 
-        super(PartitionNode, self).__init__(config['name'])
+        super(PartitionNode, self).__init__(config['name'], state)
 
         self.base = config['base']
         self.partitioning = parent
         self.prev_partition = prev_partition
 
+        # filter out some MBR only options for clarity
+        if self.partitioning.label == 'gpt':
+            if 'flags' in config and 'primary' in config['flags']:
+                raise BlockDeviceSetupException(
+                    "Primary flag not supported for GPT partitions")
+
         self.flags = set()
         if 'flags' in config:
             for f in config['flags']:
@@ -47,7 +53,10 @@ class PartitionNode(NodeBase):
             raise BlockDeviceSetupException("No size in partition" % self.name)
         self.size = config['size']
 
-        self.ptype = int(config['type'], 16) if 'type' in config else 0x83
+        if self.partitioning.label == 'gpt':
+            self.ptype = str(config['type']) if 'type' in config else '8300'
+        elif self.partitioning.label == 'mbr':
+            self.ptype = int(config['type'], 16) if 'type' in config else 83
 
     def get_flags(self):
         return self.flags
@@ -65,5 +74,15 @@ class PartitionNode(NodeBase):
             edge_from.append(self.prev_partition.name)
         return (edge_from, edge_to)
 
-    def create(self, result, rollback):
-        self.partitioning.create(result, rollback)
+    # These all call back to the parent "partitioning" object to do
+    # the real work.  Every node calls it, but only one will succeed;
+    # see the gating we do in the parent function.
+    #
+    # XXX: A better model here would be for the parent object to a
+    # real node in the config graph, so it's create() gets called.
+    # These can then just be stubs.
+    def create(self):
+        self.partitioning.create()
+
+    def cleanup(self):
+        self.partitioning.cleanup()
